<?php
/**
 * BenutzerStatistik_Helper.class.php
 *
 * Helperklasse fr die Benutzerstatistik.
 *
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @copyright IBIT 2007, 2008
 * @version 1.0.0
 */

class BenutzerStatistik_Helper
{
	/**
	 * This function parses a given list of user_agents and returns a list of all
	 * used operating systems including their quantity.
	 *
	 * @param array $user_agents The list of user_agents as an array [<name>, <quantity>]{0..x}
	 * @param integer $slice Tells the function to only use a slice of the result (and to order the result in descending order)
	 * @return array The list of used operating systems as an array [<name>, <quantity>]{0..x}
	 * @static
	 */
	public static function parseOperatingSystems($user_agents, $result=array(), $slice=10)
	{
		$possible_os = array(
			'Windows'=>'Windows',
			'Macintosh'=>'Macintosh',
			'Linux'=>'X11'
		);

		foreach ($user_agents as $user_agent)
		{
			$quantity = $user_agent['quantity'];
			$os = _('unbekannt');
			foreach ($possible_os as $os_name=>$os2try)
			{
				if (strpos($user_agent['user_agent'], $os2try)!==false)
				{
					$os = $os_name;
					break;
				}
			}

			if (empty($result[$os]))
			{
				$result[$os] = array(
					'name'=>$os,
					'quantity'=>0
				);
			}
			$result[$os]['quantity'] += $quantity;
		}
		;

		usort($result, create_function('$a,$b', 'return $b["quantity"]-$a["quantity"];'));

		return $slice>0
			? array_slice($result, 0, $slice)
			: $result;
	}

	/**
	 *
	 * This function parses a given list of user_agents and returns a list of all
	 * used user agents including their version and quantity.
	 *
	 * @param array $user_agents The list of user_agents as an array [<name>, <quantity>]{0..x}
	 * @param integer $slice Tells the function to only use a slice of the result (and to order the result in descending order)
	 * @return array The list of used user agents as an array [<name>, [<version>, <quantity>]{1..x}]{0..x}
	 * @static
	 */
	public static function parseUserAgents($user_agents, $result=array(), $slice=10)
	{
		foreach ($user_agents as $user_agent)
		{
			$quantity = $user_agent['quantity'];
			$agent = $user_agent['user_agent'];
			$version = '?';
			if (preg_match('~(\d+(?:\.\d+)*)\s+(Safari)~i', $agent, $matches))
			{
				$agent = $matches[2];
				$version = $matches[1];
			}
			elseif (preg_match('~(Epiphany|Camino|Safari|SeaMonkey|IceWeasel|Konqueror|Netscape|Opera|Firefox|MSIE)[ /]([^;\s]+)(?:;|\s|$)~iS', $agent, $matches))
			{
				$agent = $matches[1];
				$version = $matches[2];
			}
			else
			{
				$version = $agent;
				$agent = _('unbekannt');
			}

			$hash = md5($agent);
			if (!empty($result[$hash]))
			{
				$result[$hash]['quantity'] += $quantity;
				if (empty($result[$hash]['versions'][$version]))
				{
					$result[$hash]['versions'][$version] = 0;
				}
				$result[$hash]['versions'][$version] += $quantity;
			}
			else
			{
				$result[$hash] = array(
					'name'=>$agent,
					'versions'=>array(
						$version=>$quantity
					),
					'quantity'=>$quantity
				);
			}
		}

		foreach ($result as $index=>$item)
		{
			uasort($item['versions'], create_function('$a, $b', 'return $b-$a;'));
			$result[$index] = $item;
		}
		usort($result, create_function('$a, $b', 'return $b["quantity"]-$a["quantity"];'));

		return $slice>0 ? array_slice($result, 0, $slice) : $result;
	}

	/**
	 * This function parses a given list of user_agents and returns a list of all
	 * used user agents including their version and quantity.
	 *
	 * Specialized version of the above function for aggregation purposes.
	 *
	 * @param array $user_agents The list of user_agents as an array [<name>, <quantity>]{0..x}
	 * @return array The list of used user agents as an array [<name>, [<version>, <quantity>]{1..x}]{0..x}
	 * @static
	 * @see BenutzerStatistik_Helper::parseUserAgents
	 */
	public static function parseUserAgentsForSummary($user_agents)
	{
		$result = array();
		foreach ($user_agents as $user_agent)
		{
			$quantity = $user_agent['quantity'];
			$agent = $user_agent['user_agent'];
			$version = '?';
			if (preg_match('~(\d+(?:\.\d+)*)\s+(Safari)~i', $agent, $matches))
			{
				$agent = $matches[2];
				$version = $matches[1];
			}
			elseif (preg_match('~(Epiphany|Camino|Safari|SeaMonkey|IceWeasel|Konqueror|Netscape|Opera|Firefox|MSIE)[ /]([^;\s]+)(?:;|\s|$)~iS', $agent, $matches))
			{
				$agent = $matches[1];
				$version = $matches[2];
			}
			else
			{
				$version = $agent;
				$agent = _('unbekannt');
			}

			$hash = md5($agent);
			if (!empty($result[$hash]))
			{
				if (empty($result[$hash]['versions'][$version]))
				{
					$result[$hash]['versions'][$version] = array(
						'user_agent'=>$user_agent['user_agent'],
						'quantity'=>0
					);
				}
				$result[$hash]['versions'][$version]['quantity'] += $quantity;
			}
			else
			{
				$result[$hash] = array(
					'name'=>$agent,
					'versions'=>array(
						$version=>array(
							'user_agent'=>$user_agent['user_agent'],
							'quantity'=>$quantity
						)
					)
				);
			}
		}

		return $result;
	}
}
?>