<?php
/**
 * BenutzerStatistik_Summarizer.class.php
 *
 * Diese Klasse stellt den Aggregator der Benutzerstatistik dar.
 *
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @copyright IBIT 2007, 2008
 * @version 1.0.1
 */
class BenutzerStatistik_Summarizer
{
	private $db;
	private $user_agents = null;

	/**
	 * Constructor as well as main logic
	 */
	public function __construct()
	{
		$this->db = new DB_Seminar;

		$this->summarizeScreensizes();
		$this->summarizeOperatingSystems();
		$this->summarizeBrowsers();
		$this->summarizeDaily();
		$this->summarizeMonthly();
		$this->summarizeUsers();
		$this->summarizeUsersMonthly();

		$this->cleanUp();
	}

	/**
	 * This function is used to summarize the screen sizes
	 */
	private function summarizeScreensizes()
	{
		$this->db->queryf("INSERT INTO user_statistics_screensizes (width, height, quantity) SELECT screen_width AS width, screen_height AS height, COUNT(*) AS quantity FROM user_statistics WHERE daystamp<=LAST_DAY(TIMESTAMPADD(MONTH, -1, NOW())) AND daystamp<=TIMESTAMPADD(DAY, -2, NOW()) GROUP BY screen_width, screen_height ON DUPLICATE KEY UPDATE quantity=quantity+VALUES(quantity)");
	}

	/**
	 * This function is used to summarize the operating systems
	 */
	private function summarizeOperatingSystems()
	{
		$this->readUserAgents();

		foreach (BenutzerStatistik_Helper::parseOperatingSystems($this->user_agents, array(), 0) as $os)
		{
			$this->db->queryf("INSERT INTO user_statistics_os (os, quantity) VALUES ('%s', %d) ON DUPLICATE KEY UPDATE quantity=quantity+VALUES(quantity)", $os['name'], $os['quantity']);
		}
	}

	/**
	 * This function aggregates is used to summarize the used browsers
	 */
	private function summarizeBrowsers()
	{
		$this->readUserAgents();

		$values = array();
		foreach (BenutzerStatistik_Helper::parseUserAgentsForSummary($this->user_agents) as $user_agent)
		{
			if ($user_agent['name']==_('unbekannt'))
			{
				foreach ($user_agent['versions'] as $version=>$info)
				{
					array_push($values, sprintf("(MD5('%s'), NULL, NULL, '%s', %d)",
						$info['user_agent'],
						$info['user_agent'],
						$info['quantity']
					));
				}
			}
			else
			{
				foreach ($user_agent['versions'] as $version=>$info)
				{
					array_push($values, sprintf("(MD5('%s'), '%s', '%s', '%s', %d)",
						$info['user_agent'],
						$user_agent['name'],
						$version,
						$info['user_agent'],
						$info['quantity']
					));
				}
			}
		}
		if (count($values)>0)
		{
			$this->db->queryf("INSERT INTO user_statistics_browser (agent_hash, browser, version, user_agent, quantity) VALUES %s ON DUPLICATE KEY UPDATE quantity=quantity+VALUES(quantity)", implode(',', $values));
		}
	}

	/**
	 * Summarizes data from statistics table into another table containing summarized daily data.
	 */
	private function summarizeDaily()
	{
		$this->db->queryf("INSERT INTO user_statistics_daily (daystamp, permission, visits, unique_visits, hits) SELECT daystamp, permission, COUNT(*) AS visits, COUNT(DISTINCT hash) AS unique_visits, SUM(hits) AS hits FROM user_statistics WHERE daystamp<=LAST_DAY(TIMESTAMPADD(MONTH, -1, NOW())) AND daystamp<=TIMESTAMPADD(DAY, -2, NOW()) GROUP BY daystamp, permission");
	}

	/**
	 * Summarizes data from statistics table into another table containing summarized monthly data.
	 */
	private function summarizeMonthly()
	{
		$this->db->queryf("INSERT INTO user_statistics_monthly (month_stamp, permission, visits, unique_visits, hits, javascript) (SELECT LAST_DAY(daystamp) AS month_stamp, permission, COUNT(*) AS visits, COUNT(DISTINCT hash) AS unique_visits, SUM(hits) AS hits, SUM(javascript) AS javascript FROM user_statistics WHERE daystamp<=LAST_DAY(TIMESTAMPADD(MONTH, -1, NOW())) AND daystamp<=TIMESTAMPADD(DAY, -2, NOW()) GROUP BY LAST_DAY(daystamp), permission) ON DUPLICATE KEY UPDATE visits=visits+VALUES(visits), unique_visits=unique_visits+VALUES(unique_visits), hits=hits+VALUES(hits), javascript=javascript+VALUES(javascript)");
	}

	/**
	 * Summarizes data about the uniqueness of users into another table.
	 */
	private function summarizeUsers()
	{
		$this->db->queryf("INSERT INTO user_statistics_uniqueusers (hash, permission, hits, visits) (SELECT hash, permission, SUM(hits) AS hits, COUNT(*) AS visits FROM user_statistics WHERE daystamp<=LAST_DAY(TIMESTAMPADD(MONTH, -1, NOW())) AND daystamp<=TIMESTAMPADD(DAY, -2, NOW()) GROUP BY hash) ON DUPLICATE KEY UPDATE hits=hits+VALUES(hits), visits=visits+VALUES(visits)");
	}

	/**
	 * Summarizes data about quarterly unique users.
	 */
	private function summarizeUsersMonthly()
	{
		$this->db->queryf("INSERT IGNORE INTO user_statistics_uniqueusers_monthly (hash, permission, month, year) (SELECT DISTINCT hash, permission, MONTH(daystamp) AS month, YEAR(daystamp) AS year FROM user_statistics WHERE daystamp<=LAST_DAY(TIMESTAMPADD(MONTH, -1, NOW())) AND daystamp<=TIMESTAMPADD(DAY, -2, NOW()))");
	}

	/**
	 * Reads all required user agents from database
	 */
	private function readUserAgents()
	{
		if ($this->user_agents!==null)
		{
			return;
		}

		$this->user_agents = array();
		$this->db->queryf("SELECT user_agent, COUNT(*) AS quantity FROM user_statistics WHERE daystamp<=LAST_DAY(TIMESTAMPADD(MONTH, -1, NOW())) AND daystamp<=TIMESTAMPADD(DAY, -2, NOW()) GROUP BY user_agent");
		while ($this->db->next_record())
		{
			array_push($this->user_agents, array(
				'user_agent'=>$this->db->f('user_agent'),
				'quantity'=>$this->db->f('quantity')
			));
		}
		$this->db->free();
	}

	/**
	 * This function removes all summarized from the statistics table and optimizes the table
	 */
	private function cleanUp()
	{
		$this->db->queryf("DELETE FROM user_statistics WHERE daystamp<=LAST_DAY(TIMESTAMPADD(MONTH, -1, NOW())) AND daystamp<=TIMESTAMPADD(DAY, -2, NOW())");
		$this->db->queryf("OPTIMIZE TABLE user_statistics");
	}
}
?>